#' Estimates mitigated fraction from clustered or stratified data.
#' 
#' Averages the U statistic over the clusters and computes MF from it. Clusters are excluded if they do not include both treatments.
#' 
#' @title Clustered mitigated fraction
#' @usage MFClus(formula, data, compare = c("con", "vac"), trace.it = FALSE)
#' @param formula Formula of the form \code{y ~ x + cluster(w)}, where y is a continuous response, x is a factor with two levels of treatment, and w is a factor indicating the clusters.
#' @param data Data frame
#' @param compare Text vector stating the factor levels - \code{compare[1]} is the control or reference group to which \code{compare[2]} is compared
#' @param trace.it Verbose tracking of the cycles? Default FALSE.
#' @return a \code{\link{mfcluster-class}} data object
#' @export
#' @references Siev D. (2005). An estimator of intervention effect on disease severity. \emph{Journal of Modern Applied Statistical Methods.} \bold{4:500--508}
#' @author David Siev \email{david.siev@@aphis.usda.gov}
#' @seealso \code{\link{mfcluster-class}}
#' @examples
#' \dontrun{
#' MFClus(lesion ~ group + cluster(litter), piglung)
#' 
#' #  Comparing vac to con 
#' #  
#' #  MF = 0.3533835 
#' #  
#' #  By Cluster
#' #     w  u         r n1 n2         mf
#' #  U 25 10 0.4000000  5  5 -0.2000000
#' #  K 12  2 0.2500000  4  2 -0.5000000
#' #  Z 16 10 0.8333333  3  4  0.6666667
#' #  D  3  2 1.0000000  1  2  1.0000000
#' #  N  1  0 0.0000000  1  3 -1.0000000
#' #  T  8  5 0.8333333  2  3  0.6666667
#' #  P  4  1 0.5000000  2  1  0.0000000
#' #  L  3  2 0.6666667  1  3  0.3333333
#' #  G 15  9 0.7500000  3  4  0.5000000
#' #  J 15  9 1.0000000  3  3  1.0000000
#' #  W  6  3 0.7500000  2  2  0.5000000
#' #  A  9  3 0.3333333  3  3 -0.3333333
#' #  X 12  6 1.0000000  3  2  1.0000000
#' #  F 13  7 0.7777778  3  3  0.5555556
#' #  S 21 11 0.9166667  4  3  0.8333333
#' #  H 14  8 0.8888889  3  3  0.7777778
#' #  Y  2  1 1.0000000  1  1  1.0000000
#' #  E  2  1 1.0000000  1  1  1.0000000
#' #  
#' #  All
#' #        w  u         r n1 n2        mf
#' #  All 181 90 0.6766917 50 52 0.3533835
#' #  
#' #  Excluded Clusters
#' #  [1] M, Q, R, B, O, V, I, C
#' }
##
##--------------------------------------------------------------------
## Clustered or Stratified MF
##--------------------------------------------------------------------
##
MFClus <- function(formula, data, compare = c("con", "vac"), trace.it = FALSE){
    # formula of the form response ~ treatment + cluster(clustername)
    # based on prob{F(y)<F(x)}
    # within-cluster ranking only 
    # 3/19/01 initial coding
    # revised 10/3/06 to eliminate clusters without both treatments represented
    
   # assign('cluster', function(x) {return(x)}, envir = parent.env(environment()))
	cluster <- function(x) {return(x)}
    this.call <- match.call()
    Terms <- terms(formula, specials = 'cluster', data = data)
	environment(Terms) <- environment()

    A <- model.frame(formula = Terms, data = data)

    dat <- A[, 1]
    group <- A[, 2]
## remove group levels that aren't present; don't want to eval for empty groups - mcv 08/27/13
	group <- factor(group)
    clusters <- A[, 3]
    
    strat <- unique(as.character(clusters))
    id <- compare
    out <- matrix(NA, length(strat), 6, dimnames = list(strat, c("w", "u", "r", 
		"n1", "n2", "mf")))
    excluded.clusters <- NULL
    for(stratum in strat) {     
        x <- dat[group == id[1] & as.character(clusters) == stratum]
        y <- dat[group == id[2] & as.character(clusters) == stratum]
        n.x <- length(x)
        n.y <- length(y)
        if(n.x > 0 & n.y > 0) {
            N <- n.x + n.y
            x.y <- c(x, y)
            w <- sum(rank(x.y)[1:n.x])
            u <- w - (n.x * (n.x + 1))/2
            r <- u/(n.x * n.y)
            mf <- 2 * r - 1
            out[stratum,  ] <- c(w, u, r, n.x, n.y, mf)
        }
        else {
            if(trace.it) cat('Cluster',stratum,'missing a treatment\n')
            out[stratum,  ] <- c(NA, NA, NA, n.x, n.y, NA)
            excluded.clusters <- c(excluded.clusters, stratum)
        }
    }
    All <- apply(out, 2, sum, na.rm = T)
    R <- All["u"]/sum(out[, "n1"] * out[, "n2"])
    MF <- 2 * R - 1
    All["r"] <- R
    All["mf"] <- MF
    All <- data.frame(t(All))
    dimnames(All)[[1]] <- 'All'
    # result <- list(All = All, byCluster = out, excludedClusters = excluded.clusters,
		# call = this.call, compare = compare)
    # class(result) <- 'mfcluster'
    # return(result)
	return(mfcluster$new(All = All, byCluster = out, excludedClusters = excluded.clusters,
		call = this.call, compare = compare))
}
